@@ -15,6 +15,8 @@ from . import transformations as tf
 from .base import Trimesh
 from .constants import log, tol
 from .geometry import align_vectors, faces_to_edges, plane_transform
+from .resources import get_json
+from .typed import ArrayLike, Dict, Integer, Number, Optional
 
 try:
     # shapely is a soft dependency
@@ -31,8 +33,17 @@ try:
 except BaseException as E:
     _tri_earcut = exceptions.ExceptionWrapper(E)
 
+# get stored values for simple box and icosahedron primitives
+_data = get_json("creation.json")
 
-def revolve(linestring, angle=None, sections=None, transform=None, **kwargs):
+
+def revolve(
+    linestring: ArrayLike,
+    angle: Optional[Number] = None,
+    sections: Optional[Integer] = None,
+    transform: Optional[ArrayLike] = None,
+    **kwargs,
+) -> Trimesh:
     """
     Revolve a 2D line string around the 2D Y axis, with a result with
     the 2D Y axis pointing along the 3D Z axis.
@@ -165,7 +176,9 @@ def revolve(linestring, angle=None, sections=None, transform=None, **kwargs):
     return mesh
 
 
-def extrude_polygon(polygon, height, transform=None, **kwargs):
+def extrude_polygon(
+    polygon: "Polygon", height: Number, transform: Optional[ArrayLike] = None, **kwargs
+) -> Trimesh:
     """
     Extrude a 2D shapely polygon into a 3D mesh
 
@@ -175,6 +188,8 @@ def extrude_polygon(polygon, height, transform=None, **kwargs):
       2D geometry to extrude
     height : float
       Distance to extrude polygon along Z
+    transform : None or (4, 4) float
+      Transform to apply to mesh after construction
     triangle_args : str or None
       Passed to triangle
     **kwargs : dict
@@ -194,11 +209,23 @@ def extrude_polygon(polygon, height, transform=None, **kwargs):
     return mesh
 
 
-def sweep_polygon(polygon, path, angles=None, **kwargs):
+def sweep_polygon(
+    polygon: "Polygon",
+    path: ArrayLike,
+    angles: Optional[ArrayLike] = None,
+    cap: bool = True,
+    connect: bool = True,
+    kwargs: Optional[Dict] = None,
+    **triangulation,
+) -> Trimesh:
     """
-    Extrude a 2D shapely polygon into a 3D mesh along an
-    arbitrary 3D path. Doesn't handle sharp curvature well.
+    Extrude a 2D polygon into a 3D mesh along a 3D path. Note that this
+    does *not* handle the case where there is very sharp curvature leading
+    the polygon to intersect the plane of a previous slice, and does *not*
+    scale the polygon along the induced normal to result in a constant cross section.
 
+    You may want to resample your path with a B-spline, i.e:
+      `trimesh.path.simplify.resample_spline(path, smooth=0.2, count=100)`
 
     Parameters
     ----------
@@ -206,11 +233,19 @@ def sweep_polygon(polygon, path, angles=None, **kwargs):
       Profile to sweep along path
     path : (n, 3) float
       A path in 3D
-    angles :  (n,) float
+    angles : (n,) float
       Optional rotation angle relative to prior vertex
-      at each vertex
-    **kwargs : dict
-      Passed to `triangulate_polygon`.
+      at each vertex.
+    cap
+      If an open path is passed apply a cap to both ends.
+    connect
+      If a closed path is passed connect the sweep into
+      a single watertight mesh.
+    kwargs : dict
+      Passed to the mesh constructor.
+    **triangulation
+      Passed to `triangulate_polygon`, i.e. `engine='triangle'`
+
     Returns
     -------
     mesh : trimesh.Trimesh
@@ -221,85 +256,184 @@ def sweep_polygon(polygon, path, angles=None, **kwargs):
     if not util.is_shape(path, (-1, 3)):
         raise ValueError("Path must be (n, 3)!")
 
+    if angles is not None:
+        angles = np.asanyarray(angles, dtype=np.float64)
+        if angles.shape != (len(path),):
+            raise ValueError(angles.shape)
+    else:
+        # set all angles to zero
+        angles = np.zeros(len(path), dtype=np.float64)
+
+    # check to see if path is closed i.e. first and last vertex are the same
+    closed = np.linalg.norm(path[0] - path[-1]) < tol.merge
     # Extract 2D vertices and triangulation
-    verts_2d = np.array(polygon.exterior.coords)[:-1]
-    base_verts_2d, faces_2d = triangulate_polygon(polygon, **kwargs)
-    n = len(verts_2d)
+    vertices_2D, faces_2D = triangulate_polygon(polygon, **triangulation)
 
-    # Create basis for first planar polygon cap
-    x, y, z = util.generate_basis(path[0] - path[1])
-    tf_mat = np.ones((4, 4))
-    tf_mat[:3, :3] = np.c_[x, y, z]
-    tf_mat[:3, 3] = path[0]
+    # stack the `(n, 3)` faces into `(3 * n, 2)` edges
+    edges = faces_to_edges(faces_2D)
+    # edges which only occur once are on the boundary of the polygon
+    # since the triangulation may have subdivided the boundary of the
+    # shapely polygon, we need to find it again
+    edges_unique = grouping.group_rows(np.sort(edges, axis=1), require_count=1)
+    # subset the vertices to only ones included in the boundary
+    unique, inverse = np.unique(edges[edges_unique].reshape(-1), return_inverse=True)
+    # take only the vertices in the boundary
+    # and stack them with zeros and ones so we can use dot
+    # products to transform them all over the place
+    vertices_tf = np.column_stack(
+        (vertices_2D[unique], np.zeros(len(unique)), np.ones(len(unique)))
+    )
+    # the indices of vertices_tf
+    boundary = inverse.reshape((-1, 2))
 
-    # Compute 3D locations of those vertices
-    verts_3d = np.c_[verts_2d, np.zeros(n)]
-    verts_3d = tf.transform_points(verts_3d, tf_mat)
-    base_verts_3d = np.c_[base_verts_2d, np.zeros(len(base_verts_2d))]
-    base_verts_3d = tf.transform_points(base_verts_3d, tf_mat)
+    # now create the normals for the plane each slice will lie on
+    # consider the simple path with 3 vertices and therefore 2 vectors:
+    # - the first plane will be exactly along the first vector
+    # - the second plane will be the average of the two vectors
+    # - the last plane will be exactly along the last vector
+    # and each plane origin will be the corresponding vertex on the path
+    vector = util.unitize(path[1:] - path[:-1])
+    # unitize instead of / 2 as they may be degenerate / zero
+    vector_mean = util.unitize(vector[1:] + vector[:-1])
+    # collect the vectors into plane normals
+    normal = np.concatenate([[vector[0]], vector_mean, [vector[-1]]], axis=0)
 
-    # keep matching sequence of vertices and 0- indexed faces
-    vertices = [base_verts_3d]
-    faces = [faces_2d]
+    if closed and connect:
+        # if we have a closed loop average the first and last planes
+        normal[0] = util.unitize(normal[[0, -1]].mean(axis=0))
 
-    # Compute plane normals for each turn --
-    # each turn induces a plane halfway between the two vectors
-    v1s = util.unitize(path[1:-1] - path[:-2])
-    v2s = util.unitize(path[1:-1] - path[2:])
-    norms = np.cross(np.cross(v1s, v2s), v1s + v2s)
-    norms[(norms == 0.0).all(1)] = v1s[(norms == 0.0).all(1)]
-    norms = util.unitize(norms)
-    final_v1 = util.unitize(path[-1] - path[-2])
-    norms = np.vstack((norms, final_v1))
-    v1s = np.vstack((v1s, final_v1))
+    # planes should have one unit normal and one vertex each
+    assert normal.shape == path.shape
 
-    # Create all side walls by projecting the 3d vertices into each plane
-    # in succession
-    for i in range(len(norms)):
-        verts_3d_prev = verts_3d
+    # get the spherical coordinates for the normal vectors
+    theta, phi = util.vector_to_spherical(normal).T
 
-        # Rotate if needed
-        if angles is not None:
-            tf_mat = tf.rotation_matrix(angles[i], norms[i], path[i])
-            verts_3d_prev = tf.transform_points(verts_3d_prev, tf_mat)
+    # collect the trig values into numpy arrays we can compose into matrices
+    cos_theta, sin_theta = np.cos(theta), np.sin(theta)
+    cos_phi, sin_phi = np.cos(phi), np.sin(phi)
+    cos_roll, sin_roll = np.cos(angles), np.sin(angles)
 
-        # Project vertices onto plane in 3D
-        ds = np.einsum("ij,j->i", (path[i + 1] - verts_3d_prev), norms[i])
-        ds = ds / np.dot(v1s[i], norms[i])
+    # we want a rotation which will be the identity for a Z+ vector
+    # this was constructed and unrolled from the following sympy block
+    # theta, phi, roll = sp.symbols("theta phi roll")
+    # matrix = (
+    #     tf.rotation_matrix(roll, [0, 0, 1]) @
+    #     tf.rotation_matrix(phi, [1, 0, 0]) @
+    #     tf.rotation_matrix((sp.pi / 2) - theta, [0, 0, 1])
+    # ).inv()
+    # matrix.simplify()
 
-        verts_3d_new = np.einsum("i,j->ij", ds, v1s[i]) + verts_3d_prev
+    # shorthand for stacking
+    zeros = np.zeros(len(theta))
+    ones = np.ones(len(theta))
 
-        # Add to face and vertex lists
-        new_faces = [[i + n, (i + 1) % n, i] for i in range(n)]
-        new_faces.extend([[(i - 1) % n + n, i + n, i] for i in range(n)])
+    # stack initially as one unrolled matrix per row
+    transforms = np.column_stack(
+        [
+            -sin_roll * cos_phi * cos_theta + sin_theta * cos_roll,
+            sin_roll * sin_theta + cos_phi * cos_roll * cos_theta,
+            sin_phi * cos_theta,
+            path[:, 0],
+            -sin_roll * sin_theta * cos_phi - cos_roll * cos_theta,
+            -sin_roll * cos_theta + sin_theta * cos_phi * cos_roll,
+            sin_phi * sin_theta,
+            path[:, 1],
+            sin_phi * sin_roll,
+            -sin_phi * cos_roll,
+            cos_phi,
+            path[:, 2],
+            zeros,
+            zeros,
+            zeros,
+            ones,
+        ]
+    ).reshape((-1, 4, 4))
 
-        # save faces and vertices into a sequence
-        faces.append(np.array(new_faces))
-        vertices.append(np.vstack((verts_3d, verts_3d_new)))
+    if tol.strict:
+        # make sure that each transform moves the Z+ vector to the requested normal
+        for n, matrix in zip(normal, transforms):
+            check = tf.transform_points([[0.0, 0.0, 1.0]], matrix, translate=False)[0]
+            assert np.allclose(check, n)
 
-        verts_3d = verts_3d_new
+    # apply transforms to prebaked homogeneous coordinates
+    vertices_3D = np.concatenate(
+        [np.dot(vertices_tf, matrix.T) for matrix in transforms], axis=0
+    )[:, :3]
 
-    # do the main stack operation from a sequence to (n,3) arrays
-    # doing one vstack provides a substantial speedup by
-    # avoiding a bunch of temporary  allocations
-    vertices, faces = util.append_faces(vertices, faces)
+    # now construct the faces with one group of boundary faces per slice
+    stride = len(unique)
+    boundary_next = boundary + stride
+    faces_slice = np.column_stack(
+        [boundary, boundary_next[:, :1], boundary_next[:, ::-1], boundary[:, 1:]]
+    ).reshape((-1, 3))
 
-    # Create final cap
-    x, y, z = util.generate_basis(path[-1] - path[-2])
-    vecs = verts_3d - path[-1]
-    coords = np.c_[np.einsum("ij,j->i", vecs, x), np.einsum("ij,j->i", vecs, y)]
-    base_verts_2d, faces_2d = triangulate_polygon(Polygon(coords), **kwargs)
-    base_verts_3d = (
-        np.einsum("i,j->ij", base_verts_2d[:, 0], x)
-        + np.einsum("i,j->ij", base_verts_2d[:, 1], y)
-    ) + path[-1]
-    faces = np.vstack((faces, faces_2d + len(vertices)))
-    vertices = np.vstack((vertices, base_verts_3d))
+    # offset the slices
+    faces = [faces_slice + offset for offset in np.arange(len(path) - 1) * stride]
 
-    return Trimesh(vertices, faces)
+    # connect only applies to closed paths
+    if closed and connect:
+        # the last slice will not be required
+        max_vertex = (len(path) - 1) * stride
+        # clip off the duplicated vertices
+        vertices_3D = vertices_3D[:max_vertex]
+        # apply the modulus in-place to a conservative subset
+        faces[-1] %= max_vertex
+    elif cap:
+        # these are indices of `vertices_2D` that were not on the boundary
+        # which can happen for triangulation algorithms that added vertices
+        # we don't currently support that but you could append the unconsumed
+        # vertices and then update the mapping below to reflect that
+        unconsumed = set(unique).difference(faces_2D.ravel())
+        if len(unconsumed) > 0:
+            raise NotImplementedError("triangulation added vertices: no logic to cap!")
+
+        # map the 2D faces to the order we used
+        mapped = np.zeros(unique.max() + 2, dtype=np.int64)
+        mapped[unique] = np.arange(len(unique))
+
+        # now should correspond to the first vertex block
+        cap_zero = mapped[faces_2D]
+        # winding will be along +Z so flip for the bottom cap
+        faces.append(np.fliplr(cap_zero))
+        # offset the end cap
+        faces.append(cap_zero + stride * (len(path) - 1))
+
+    if kwargs is None:
+        kwargs = {}
+
+    if "process" not in kwargs:
+        # we should be constructing clean meshes here
+        # so we don't need to run an expensive verex merge
+        kwargs["process"] = False
+
+    # stack the faces used
+    faces = np.concatenate(faces, axis=0)
+
+    # generate the mesh from the face data
+    mesh = Trimesh(vertices=vertices_3D, faces=faces, **kwargs)
+
+    if tol.strict:
+        # we should not have included any unused vertices
+        assert len(np.unique(faces)) == len(vertices_3D)
+
+        if cap:
+            # mesh should always be a volume if cap is true
+            assert mesh.is_volume
+
+        if closed and connect:
+            assert mesh.is_volume
+            assert mesh.body_count == 1
+
+    return mesh
 
 
-def extrude_triangulation(vertices, faces, height, transform=None, **kwargs):
+def extrude_triangulation(
+    vertices: ArrayLike,
+    faces: ArrayLike,
+    height: Number,
+    transform: Optional[ArrayLike] = None,
+    **kwargs,
+) -> Trimesh:
     """
     Extrude a 2D triangulation into a watertight mesh.
 
@@ -311,6 +445,8 @@ def extrude_triangulation(vertices, faces, height, transform=None, **kwargs):
       Triangle indexes of vertices
     height : float
       Distance to extrude triangulation
+    transform : None or (4, 4) float
+      Transform to apply to mesh after construction
     **kwargs : dict
       Passed to Trimesh constructor
 
@@ -383,10 +519,13 @@ def extrude_triangulation(vertices, faces, height, transform=None, **kwargs):
     return mesh
 
 
-def triangulate_polygon(polygon, triangle_args=None, engine=None, **kwargs):
+def triangulate_polygon(
+    polygon, triangle_args: Optional[str] = None, engine: Optional[str] = None, **kwargs
+):
     """
     Given a shapely polygon create a triangulation using a
-    python interface to `triangle.c` or mapbox-earcut.
+    python interface to the permissively licensed `mapbox-earcut`
+    or the more robust `triangle.c`.
     > pip install triangle
     > pip install mapbox_earcut
 
@@ -395,9 +534,9 @@ def triangulate_polygon(polygon, triangle_args=None, engine=None, **kwargs):
     polygon : Shapely.geometry.Polygon
         Polygon object to be triangulated.
     triangle_args : str or None
-        Passed to triangle.triangulate i.e: 'p', 'pq30'
+        Passed to triangle.triangulate i.e: 'p', 'pq30', 'pY'="don't insert vert"
     engine : None or str
-      Any value other than 'earcut' will use `triangle`
+      None or 'earcut' will use earcut, 'triangle' will use triangle
 
     Returns
     --------------
@@ -446,7 +585,7 @@ def triangulate_polygon(polygon, triangle_args=None, engine=None, **kwargs):
         raise ValueError("no valid triangulation engine!")
 
 
-def _polygon_to_kwargs(polygon):
+def _polygon_to_kwargs(polygon) -> Dict:
     """
     Given a shapely polygon generate the data to pass to
     the triangle mesh generator
@@ -538,13 +677,18 @@ def _polygon_to_kwargs(polygon):
     return result
 
 
-def box(extents=None, transform=None, bounds=None, **kwargs):
+def box(
+    extents: Optional[ArrayLike] = None,
+    transform: Optional[ArrayLike] = None,
+    bounds: Optional[ArrayLike] = None,
+    **kwargs,
+):
     """
     Return a cuboid.
 
     Parameters
     ------------
-    extents : float, or (3,) float
+    extents : (3,) float
       Edge lengths
     transform: (4, 4) float
       Transformation matrix
@@ -558,12 +702,10 @@ def box(extents=None, transform=None, bounds=None, **kwargs):
     geometry : trimesh.Trimesh
       Mesh of a cuboid
     """
-    # vertices of the cube
-    vertices = np.array(
-        [0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1],
-        order="C",
-        dtype=np.float64,
-    ).reshape((-1, 3))
+    # vertices of the cube from reference
+    vertices = np.array(_data["box"]["vertices"], order="C", dtype=np.float64)
+    faces = np.array(_data["box"]["faces"], order="C", dtype=np.int64)
+    face_normals = np.array(_data["box"]["face_normals"], order="C", dtype=np.float64)
 
     # resize cube based on passed extents
     if bounds is not None:
@@ -572,7 +714,7 @@ def box(extents=None, transform=None, bounds=None, **kwargs):
         bounds = np.array(bounds, dtype=np.float64)
         if bounds.shape != (2, 3):
             raise ValueError("`bounds` must be (2, 3) float!")
-        extents = bounds.ptp(axis=0)
+        extents = np.ptp(bounds, axis=0)
         vertices *= extents
         vertices += bounds[0]
     elif extents is not None:
@@ -585,87 +727,6 @@ def box(extents=None, transform=None, bounds=None, **kwargs):
         vertices -= 0.5
         extents = np.asarray((1.0, 1.0, 1.0), dtype=np.float64)
 
-    # hardcoded face indices
-    faces = [
-        1,
-        3,
-        0,
-        4,
-        1,
-        0,
-        0,
-        3,
-        2,
-        2,
-        4,
-        0,
-        1,
-        7,
-        3,
-        5,
-        1,
-        4,
-        5,
-        7,
-        1,
-        3,
-        7,
-        2,
-        6,
-        4,
-        2,
-        2,
-        7,
-        6,
-        6,
-        5,
-        4,
-        7,
-        5,
-        6,
-    ]
-    faces = np.array(faces, order="C", dtype=np.int64).reshape((-1, 3))
-
-    face_normals = [
-        -1,
-        0,
-        0,
-        0,
-        -1,
-        0,
-        -1,
-        0,
-        0,
-        0,
-        0,
-        -1,
-        0,
-        0,
-        1,
-        0,
-        -1,
-        0,
-        0,
-        0,
-        1,
-        0,
-        1,
-        0,
-        0,
-        0,
-        -1,
-        0,
-        1,
-        0,
-        1,
-        0,
-        0,
-        1,
-        0,
-        0,
-    ]
-    face_normals = np.asanyarray(face_normals, order="C", dtype=np.float64).reshape(-1, 3)
-
     if "metadata" not in kwargs:
         kwargs["metadata"] = {}
     kwargs["metadata"].update({"shape": "box", "extents": extents})
@@ -681,7 +742,7 @@ def box(extents=None, transform=None, bounds=None, **kwargs):
     return box
 
 
-def icosahedron(**kwargs):
+def icosahedron(**kwargs) -> Trimesh:
     """
     Create an icosahedron, one of the platonic solids which is has 20 faces.
 
@@ -695,117 +756,15 @@ def icosahedron(**kwargs):
     ico : trimesh.Trimesh
       Icosahederon centered at the origin.
     """
-    t = (1.0 + 5.0**0.5) / 2.0
-    vertices = [
-        -1,
-        t,
-        0,
-        1,
-        t,
-        0,
-        -1,
-        -t,
-        0,
-        1,
-        -t,
-        0,
-        0,
-        -1,
-        t,
-        0,
-        1,
-        t,
-        0,
-        -1,
-        -t,
-        0,
-        1,
-        -t,
-        t,
-        0,
-        -1,
-        t,
-        0,
-        1,
-        -t,
-        0,
-        -1,
-        -t,
-        0,
-        1,
-    ]
-    faces = [
-        0,
-        11,
-        5,
-        0,
-        5,
-        1,
-        0,
-        1,
-        7,
-        0,
-        7,
-        10,
-        0,
-        10,
-        11,
-        1,
-        5,
-        9,
-        5,
-        11,
-        4,
-        11,
-        10,
-        2,
-        10,
-        7,
-        6,
-        7,
-        1,
-        8,
-        3,
-        9,
-        4,
-        3,
-        4,
-        2,
-        3,
-        2,
-        6,
-        3,
-        6,
-        8,
-        3,
-        8,
-        9,
-        4,
-        9,
-        5,
-        2,
-        4,
-        11,
-        6,
-        2,
-        10,
-        8,
-        6,
-        7,
-        9,
-        8,
-        1,
-    ]
-    # scale vertices so each vertex radius is 1.0
-    vertices = np.reshape(vertices, (-1, 3)) / np.sqrt(2.0 + t)
-    faces = np.reshape(faces, (-1, 3))
-
+    # get stored pre-baked primitive values
+    vertices = np.array(_data["icosahedron"]["vertices"], dtype=np.float64)
+    faces = np.array(_data["icosahedron"]["faces"], dtype=np.int64)
     return Trimesh(
         vertices=vertices, faces=faces, process=kwargs.pop("process", False), **kwargs
     )
 
 
-def icosphere(subdivisions=3, radius=1.0, **kwargs):
+def icosphere(subdivisions: Integer = 3, radius: Number = 1.0, **kwargs):
     """
     Create an isophere centered at the origin.
 
@@ -825,9 +784,12 @@ def icosphere(subdivisions=3, radius=1.0, **kwargs):
     ico : trimesh.Trimesh
       Meshed sphere
     """
+    radius = float(radius)
+    subdivisions = int(subdivisions)
 
     ico = icosahedron()
     ico._validate = False
+
     for _ in range(subdivisions):
         ico = ico.subdivide()
         vectors = ico.vertices
@@ -835,6 +797,13 @@ def icosphere(subdivisions=3, radius=1.0, **kwargs):
         unit = vectors / scalar.reshape((-1, 1))
         ico.vertices += unit * (radius - scalar).reshape((-1, 1))
 
+    # if we didn't subdivide we still need to refine the radius
+    if subdivisions <= 0:
+        vectors = ico.vertices
+        scalar = np.sqrt(np.dot(vectors**2, [1, 1, 1]))
+        unit = vectors / scalar.reshape((-1, 1))
+        ico.vertices += unit * (radius - scalar).reshape((-1, 1))
+
     if "color" in kwargs:
         warnings.warn(
             "`icosphere(color=...)` is deprecated and will "
@@ -854,7 +823,12 @@ def icosphere(subdivisions=3, radius=1.0, **kwargs):
     )
 
 
-def uv_sphere(radius=1.0, count=None, transform=None, **kwargs):
+def uv_sphere(
+    radius: Number = 1.0,
+    count: Optional[ArrayLike] = None,
+    transform: Optional[ArrayLike] = None,
+    **kwargs,
+) -> Trimesh:
     """
     Create a UV sphere (latitude + longitude) centered at the
     origin. Roughly one order of magnitude faster than an
@@ -866,6 +840,8 @@ def uv_sphere(radius=1.0, count=None, transform=None, **kwargs):
       Radius of sphere
     count : (2,) int
       Number of latitude and longitude lines
+    transform : None or (4, 4) float
+      Transform to apply to mesh after construction
     kwargs : dict
       Passed thgrough
     Returns
@@ -896,7 +872,12 @@ def uv_sphere(radius=1.0, count=None, transform=None, **kwargs):
     )
 
 
-def capsule(height=1.0, radius=1.0, count=None, transform=None):
+def capsule(
+    height: Number = 1.0,
+    radius: Number = 1.0,
+    count: Optional[ArrayLike] = None,
+    transform: Optional[ArrayLike] = None,
+):
     """
     Create a mesh of a capsule, or a cylinder with hemispheric ends.
 
@@ -908,7 +889,8 @@ def capsule(height=1.0, radius=1.0, count=None, transform=None):
       Radius of the cylinder and hemispheres
     count : (2,) int
       Number of sections on latitude and longitude
-
+    transform : None or (4, 4) float
+      Transform to apply to mesh after construction
     Returns
     ----------
     capsule : trimesh.Trimesh
@@ -943,7 +925,13 @@ def capsule(height=1.0, radius=1.0, count=None, transform=None):
     )
 
 
-def cone(radius, height, sections=None, transform=None, **kwargs):
+def cone(
+    radius: Number,
+    height: Number,
+    sections: Optional[Integer] = None,
+    transform: Optional[ArrayLike] = None,
+    **kwargs,
+) -> Trimesh:
     """
     Create a mesh of a cone along Z centered at the origin.
 
@@ -978,7 +966,14 @@ def cone(radius, height, sections=None, transform=None, **kwargs):
     return cone
 
 
-def cylinder(radius, height=None, sections=None, segment=None, transform=None, **kwargs):
+def cylinder(
+    radius: Number,
+    height: Optional[Number] = None,
+    sections: Optional[Integer] = None,
+    segment: Optional[ArrayLike] = None,
+    transform: Optional[ArrayLike] = None,
+    **kwargs,
+):
     """
     Create a mesh of a cylinder along Z centered at the origin.
 
@@ -987,13 +982,13 @@ def cylinder(radius, height=None, sections=None, segment=None, transform=None, *
     radius : float
       The radius of the cylinder
     height : float or None
-      The height of the cylinder
+      The height of the cylinder, or None if `segment` has been passed.
     sections : int or None
       How many pie wedges should the cylinder have
     segment : (2, 3) float
       Endpoints of axis, overrides transform and height
-    transform : (4, 4) float
-      Transform to apply
+    transform : None or (4, 4) float
+      Transform to apply to mesh after construction
     **kwargs:
         passed to Trimesh to create cylinder
 
@@ -1023,7 +1018,13 @@ def cylinder(radius, height=None, sections=None, segment=None, transform=None, *
 
 
 def annulus(
-    r_min, r_max, height=None, sections=None, transform=None, segment=None, **kwargs
+    r_min: Number,
+    r_max: Number,
+    height: Optional[Number] = None,
+    sections: Optional[Integer] = None,
+    transform: Optional[ArrayLike] = None,
+    segment: Optional[ArrayLike] = None,
+    **kwargs,
 ):
     """
     Create a mesh of an annular cylinder along Z centered at the origin.
@@ -1089,7 +1090,7 @@ def annulus(
     return annulus
 
 
-def _segment_to_cylinder(segment):
+def _segment_to_cylinder(segment: ArrayLike):
     """
     Convert a line segment to a transform and height for a cylinder
     or cylinder-like primitive.
@@ -1123,7 +1124,7 @@ def _segment_to_cylinder(segment):
     return transform, height
 
 
-def random_soup(face_count=100):
+def random_soup(face_count: Integer = 100):
     """
     Return random triangles as a Trimesh
 
@@ -1144,11 +1145,11 @@ def random_soup(face_count=100):
 
 
 def axis(
-    origin_size=0.04,
-    transform=None,
-    origin_color=None,
-    axis_radius=None,
-    axis_length=None,
+    origin_size: Number = 0.04,
+    transform: Optional[ArrayLike] = None,
+    origin_color: Optional[ArrayLike] = None,
+    axis_radius: Optional[Number] = None,
+    axis_length: Optional[Number] = None,
 ):
     """
     Return an XYZ axis marker as a  Trimesh, which represents position
@@ -1157,10 +1158,10 @@ def axis(
 
     Parameters
     ----------
-    transform : (4, 4) float
-      Transformation matrix
     origin_size : float
       Radius of sphere that represents the origin
+    transform : (4, 4) float
+      Transformation matrix
     origin_color : (3,) float or int, uint8 or float
       Color of the origin
     axis_radius : float
@@ -1229,7 +1230,9 @@ def axis(
     return marker
 
 
-def camera_marker(camera, marker_height=0.4, origin_size=None):
+def camera_marker(
+    camera, marker_height: Number = 0.4, origin_size: Optional[Number] = None
+):
     """
     Create a visual marker for a camera object, including an axis and FOV.
 
@@ -1287,7 +1290,11 @@ def camera_marker(camera, marker_height=0.4, origin_size=None):
     return meshes
 
 
-def truncated_prisms(tris, origin=None, normal=None):
+def truncated_prisms(
+    tris: ArrayLike,
+    origin: Optional[ArrayLike] = None,
+    normal: Optional[ArrayLike] = None,
+):
     """
     Return a mesh consisting of multiple watertight prisms below
     a list of triangles, truncated by a specified plane.
@@ -1351,11 +1358,11 @@ def truncated_prisms(tris, origin=None, normal=None):
 
 
 def torus(
-    major_radius,
-    minor_radius,
-    major_sections=32,
-    minor_sections=32,
-    transform=None,
+    major_radius: Number,
+    minor_radius: Number,
+    major_sections: Integer = 32,
+    minor_sections: Integer = 32,
+    transform: Optional[ArrayLike] = None,
     **kwargs,
 ):
     """Create a mesh of a torus around Z centered at the origin.
@@ -1372,8 +1379,9 @@ def torus(
     minor_sections: int
       Number of sections around minor radius result should have
       If not specified default is 32 per revolution
-    transform: (4, 4) float
+    transform : (4, 4) float
       Transformation matrix
+
     **kwargs:
       passed to Trimesh to create torus
 
